"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var combineLatest_1 = require("rxjs/observable/combineLatest");
var operators_1 = require("rxjs/operators");
var Subject_1 = require("rxjs/Subject");
function extractRoutes(parent, routeProperty, inherit) {
    if (inherit === void 0) { inherit = false; }
    var routes = [];
    if (inherit) {
        while (parent.firstChild) {
            parent = parent.firstChild;
        }
    }
    while (parent) {
        routes.push(parent[routeProperty]);
        parent = parent.parent;
    }
    return combineLatest_1.combineLatest.apply(void 0, routes);
}
function extractValues(args, stream$) {
    return stream$.pipe(operators_1.map(function (routeValues) {
        var values = args.reduce(function (data, arg) {
            routeValues.forEach(function (value) {
                if (value && value[arg]) {
                    data[arg] = value[arg];
                }
            });
            return data;
        }, {});
        return args.length === 1 ? values[args[0]] : values;
    }));
}
function replaceNgOnInit(prototype) {
    var ngOnInit = prototype.ngOnInit;
    prototype.ngOnInit = function xxlFake() {
        var _this = this;
        if (!this.route) {
            throw (new Error(this.constructor.name + " uses a route-xxl @decorator without a 'route: ActivatedRoute' property"));
        }
        var state = this.__xxlState;
        var _loop_1 = function (routeProperty) {
            if (state.properties.hasOwnProperty(routeProperty)) {
                var items = state.properties[routeProperty];
                items.configs.forEach(function (item) {
                    if (routeProperty === 'tunnel') {
                        _this[item.key] = state.tunnel$ || (state.tunnel$ = new Subject_1.Subject());
                    }
                    else {
                        var stream$ = item.extractor(_this.route, routeProperty, item.config.inherit);
                        stream$ = extractValues(item.args, stream$);
                        if (item.config.pipe) {
                            stream$ = stream$.pipe.apply(stream$, item.config.pipe);
                        }
                        if (item.config.observable === false) {
                            stream$.subscribe(function (data) {
                                _this[item.key] = data;
                            });
                        }
                        else {
                            _this[item.key] = stream$;
                        }
                    }
                });
            }
        };
        for (var routeProperty in state.properties) {
            _loop_1(routeProperty);
        }
        ngOnInit.call(this);
    };
}
function updateState(state, cfg) {
    var property = state.properties[cfg.routeProperty] ||
        (state.properties[cfg.routeProperty] = { configs: [], args: [] });
    property.configs.push(cfg);
}
function routeDecoratorFactory(routeProperty, args, extractor) {
    var config = (typeof args[args.length - 1] === 'object' ? args.pop() : {});
    return function (prototype, key) {
        if (!args.length) {
            args = [key.replace(/\$$/, '')];
        }
        if (!prototype.ngOnInit) {
            throw (new Error(prototype.constructor.name + " uses the " + routeProperty + " @decorator without implementing 'ngOnInit'"));
        }
        var state = prototype.__xxlState || (prototype.__xxlState = { prototype: prototype, properties: {} });
        replaceNgOnInit(prototype);
        updateState(state, { args: args, config: config, extractor: extractor, key: key, prototype: prototype, routeProperty: routeProperty });
    };
}
function RouteData() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    return routeDecoratorFactory('data', args, extractRoutes);
}
exports.RouteData = RouteData;
function RouteParams() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    return routeDecoratorFactory('params', args, extractRoutes);
}
exports.RouteParams = RouteParams;
function RouteQueryParams() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    return routeDecoratorFactory('queryParams', args, function (route) { return route.queryParams.pipe(operators_1.map(function (params) { return [params]; })); });
}
exports.RouteQueryParams = RouteQueryParams;
function RouteTunnel() {
    return routeDecoratorFactory('tunnel', []);
}
exports.RouteTunnel = RouteTunnel;
